## daikon.bashrc
## Daikon initialization file for Bourne shell (bash) users.
## (This file should be kept in synch with daikon.cshrc and daikonenv.bat.)

## Wherever you source this file, you should set two environment variables:
##   DAIKONDIR      absolute pathname of the "daikon" directory
##   JAVA_HOME      absolute pathname of the directory containing the JDK
##                  (or "none" if you don't have it)
## Optionally, you may set the following environment variables:
##   DAIKONCLASS_SOURCES   to any value, if you want to run Daikon from .class
##        files, instead of the default, which is to use daikon.jar.
## You should not need to edit this file directly.

if [ -z "$JAVA_HOME" ]; then
  echo "JAVA_HOME environment variable is not set."
  echo "Please fix this before proceeding.  Aborting daikon.bashrc ."
  return 2
elif [ ! -d "$JAVA_HOME" -a "$JAVA_HOME" != "none" ]; then
  echo "JAVA_HOME is set to non-existent directory: $JAVA_HOME"
  echo "Please fix this before proceeding.  Aborting daikon.bashrc ."
  return 2
fi

if [ -z "$DAIKONDIR" ]; then
  echo "DAIKONDIR environment variable is not set."
  echo "Please fix this before proceeding.  Aborting daikon.bashrc ."
  return 2
elif [ ! -d "$DAIKONDIR" ]; then
  echo "DAIKONDIR is set to non-existent directory: $DAIKONDIR"
  echo "Please fix this before proceeding.  Aborting daikon.bashrc ."
  return 2
fi

if [ -z "$DAIKONBIN" ]; then
  if [ -d ${DAIKONDIR}/bin ]; then
    export DAIKONBIN=${DAIKONDIR}/bin
  elif [ -d ${DAIKONDIR}/scripts ]; then
    export DAIKONBIN=${DAIKONDIR}/scripts
  else
    echo "Cannot choose a value for environment variable DAIKONBIN."
    echo "Please fix this before proceeding.  Aborting daikon.bashrc ."
    return 2
  fi
fi

if [ -z "$PLUMEBIN" ]; then
  export PLUMEBIN=${DAIKONDIR}/plume-lib/bin
fi

# export DAIKONCLASS_SOURCES=1
if [ $DAIKONCLASS_SOURCES ]; then
  CPADD=${DAIKONDIR}/java
else
  CPADD=${DAIKONDIR}/daikon.jar
fi

if [ ! -z "$CLASSPATH" ]; then
  # For Cygwin we need to convert CLASSPATH to Window's format
  # BUT - we assume existing CLASSPATH is already in Window's format
  if [ "$OSTYPE" == "cygwin" ]; then
    CPADD1="`cygpath -wp ${CPADD}`"
    CPADD2="`cygpath -wp ${JAVA_HOME}/jre/lib/rt.jar:${JAVA_HOME}/lib/tools.jar`"
    export CLASSPATH="${CPADD1};${CLASSPATH};${CPADD2}"
  else
    export CLASSPATH=${CPADD}:${CLASSPATH}:${JAVA_HOME}/jre/lib/rt.jar:${JAVA_HOME}/lib/tools.jar
  fi
else
  if [ -n "$PS1" ]; then echo "Warning: daikon.bashrc is setting CLASSPATH, which was previously unset"; fi
  export CLASSPATH=${CPADD}:${JAVA_HOME}/jre/lib/rt.jar:${JAVA_HOME}/lib/tools.jar
  # For Cygwin we need to convert CLASSPATH to windows format
  if [ "$OSTYPE" == "cygwin" ]; then
    export CLASSPATH="`cygpath -wp $CLASSPATH`"
  fi
fi

## Make sure the specified JDK is first on your path
export PATH=$JAVA_HOME/bin:$PATH

## Add the Daikon binaries to your path
export PATH=${DAIKONBIN}:${PLUMEBIN}:${PATH}

## Indicate where to find Perl modules such as util_daikon.pm.
if [ $PERLLIB ]; then
  export PERLLIB=${DAIKONBIN}:${PLUMEBIN}:${PERLLIB}
else
  export PERLLIB=${DAIKONBIN}:${PLUMEBIN}
fi
